@@ -50,6 +50,8 @@
 
 #define RDP_SERVER_RESTART_DELAY_MS 3000
 #define RDP_SERVER_USER_CERT_SUBDIR "certificates"
+#define RDP_MAX_CERTIFICATE_FILE_SIZE_BYTES (50 * 1024)
+#define RDP_MAX_PRIVATE_KEY_FILE_SIZE_BYTES (50 * 1024)
 
 enum
 {
@@ -309,8 +311,10 @@ on_handle_import_certificate (GrdDBusRemoteDesktopRdpServer *rdp_server_interfac
   g_autoptr (GError) error = NULL;
   g_autofd int certificate_fd = -1;
   g_autofd int key_fd = -1;
+  GFileTest fd_test_results;
   int certificate_fd_index;
   int key_fd_index;
+  gboolean success;
 
   g_variant_get (certificate, "(sh)", &certificate_filename,
                  &certificate_fd_index);
@@ -324,6 +328,24 @@ on_handle_import_certificate (GrdDBusRemoteDesktopRdpServer *rdp_server_interfac
       return G_DBUS_METHOD_INVOCATION_HANDLED;
     }
 
+  success = grd_test_fd (certificate_fd, RDP_MAX_CERTIFICATE_FILE_SIZE_BYTES,
+                         &fd_test_results, &error);
+  if (!success)
+    {
+      g_prefix_error (&error, "Could not inspect certificate file descriptor");
+      g_dbus_method_invocation_return_gerror (invocation, error);
+      return G_DBUS_METHOD_INVOCATION_HANDLED;
+    }
+
+  if (!(fd_test_results & G_FILE_TEST_IS_REGULAR))
+    {
+      g_dbus_method_invocation_return_error (invocation,
+                                             G_IO_ERROR,
+                                             G_IO_ERROR_NOT_REGULAR_FILE,
+                                             "Invalid certificate file");
+      return G_DBUS_METHOD_INVOCATION_HANDLED;
+    }
+
   key_fd = g_unix_fd_list_get (fd_list, key_fd_index, &error);
   if (key_fd == -1)
     {
@@ -331,6 +353,24 @@ on_handle_import_certificate (GrdDBusRemoteDesktopRdpServer *rdp_server_interfac
       return G_DBUS_METHOD_INVOCATION_HANDLED;
     }
 
+  success = grd_test_fd (key_fd, RDP_MAX_PRIVATE_KEY_FILE_SIZE_BYTES,
+                         &fd_test_results, &error);
+  if (!success)
+    {
+      g_prefix_error (&error, "Could not inspect private key file descriptor");
+      g_dbus_method_invocation_return_gerror (invocation, error);
+      return G_DBUS_METHOD_INVOCATION_HANDLED;
+    }
+
+  if (!(fd_test_results & G_FILE_TEST_IS_REGULAR))
+    {
+      g_dbus_method_invocation_return_error (invocation,
+                                             G_IO_ERROR,
+                                             G_IO_ERROR_NOT_REGULAR_FILE,
+                                             "Invalid private key file");
+      return G_DBUS_METHOD_INVOCATION_HANDLED;
+    }
+
   grd_rewrite_path_to_user_data_dir (&certificate_filename,
                                      RDP_SERVER_USER_CERT_SUBDIR,
                                      "rdp-tls.crt");
@@ -342,7 +382,6 @@ on_handle_import_certificate (GrdDBusRemoteDesktopRdpServer *rdp_server_interfac
     }
 
   rdp_certificate = freerdp_certificate_new_from_file (certificate_filename);
-
   if (!rdp_certificate)
     {
       g_dbus_method_invocation_return_error (invocation,
@@ -362,7 +401,6 @@ on_handle_import_certificate (GrdDBusRemoteDesktopRdpServer *rdp_server_interfac
     }
 
   rdp_private_key = freerdp_key_new_from_file (key_filename);
-
   if (!rdp_private_key)
     {
       g_dbus_method_invocation_return_error (invocation,
