@@ -39,4 +39,12 @@ typedef enum
   GRD_VNC_SCREEN_SHARE_MODE_EXTEND,
 } GrdVncScreenShareMode;
 
+typedef enum
+{
+  GRD_RUNTIME_MODE_SCREEN_SHARE,
+  GRD_RUNTIME_MODE_HEADLESS,
+  GRD_RUNTIME_MODE_SYSTEM,
+  GRD_RUNTIME_MODE_HANDOVER,
+} GrdRuntimeMode;
+
 #endif /* GRD_ENUMS_H */
