@@ -29,10 +29,11 @@
 G_DECLARE_FINAL_TYPE (GrdRdpDisplayControl, grd_rdp_display_control,
                       GRD, RDP_DISPLAY_CONTROL, GObject)
 
-GrdRdpDisplayControl *grd_rdp_display_control_new (GrdSessionRdp *session_rdp,
-                                                   GrdRdpDvc     *rdp_dvc,
-                                                   HANDLE         vcm,
-                                                   uint32_t       max_monitor_count);
+GrdRdpDisplayControl *grd_rdp_display_control_new (GrdRdpLayoutManager *layout_manager,
+                                                   GrdSessionRdp       *session_rdp,
+                                                   GrdRdpDvc           *rdp_dvc,
+                                                   HANDLE               vcm,
+                                                   uint32_t             max_monitor_count);
 
 void grd_rdp_display_control_maybe_init (GrdRdpDisplayControl *display_control);
 
