@@ -55,8 +55,7 @@ typedef struct _RdpEvent
   struct
   {
     GrdStream *stream;
-    double x;
-    double y;
+    GrdEventPointerMotionAbs motion_abs;
   } input_ptr_motion_abs;
 
   /* RDP_EVENT_TYPE_INPUT_PTR_BUTTON */
@@ -100,6 +99,15 @@ struct _GrdRdpEventQueue
 
 G_DEFINE_TYPE (GrdRdpEventQueue, grd_rdp_event_queue, G_TYPE_OBJECT)
 
+void
+free_rdp_event (gpointer data)
+{
+  RdpEvent *rdp_event = data;
+
+  g_clear_object (&rdp_event->input_ptr_motion_abs.stream);
+  g_free (rdp_event);
+}
+
 static void
 handle_synchronization_event (GrdRdpEventQueue *rdp_event_queue)
 {
@@ -166,8 +174,7 @@ process_rdp_events (GrdRdpEventQueue *rdp_event_queue)
         case RDP_EVENT_TYPE_INPUT_PTR_MOTION_ABS:
           grd_session_notify_pointer_motion_absolute (
             session, rdp_event->input_ptr_motion_abs.stream,
-            rdp_event->input_ptr_motion_abs.x,
-            rdp_event->input_ptr_motion_abs.y);
+            &rdp_event->input_ptr_motion_abs.motion_abs);
           break;
         case RDP_EVENT_TYPE_INPUT_PTR_BUTTON:
           grd_session_notify_pointer_button (session,
@@ -182,7 +189,7 @@ process_rdp_events (GrdRdpEventQueue *rdp_event_queue)
           break;
         }
 
-      g_free (rdp_event);
+      free_rdp_event (rdp_event);
     }
 }
 
@@ -236,18 +243,16 @@ grd_rdp_event_queue_add_input_event_keyboard_keysym (GrdRdpEventQueue *rdp_event
 }
 
 void
-grd_rdp_event_queue_add_input_event_pointer_motion_abs (GrdRdpEventQueue *rdp_event_queue,
-                                                        GrdStream        *stream,
-                                                        double            x,
-                                                        double            y)
+grd_rdp_event_queue_add_input_event_pointer_motion_abs (GrdRdpEventQueue               *rdp_event_queue,
+                                                        GrdStream                      *stream,
+                                                        const GrdEventPointerMotionAbs *motion_abs)
 {
   RdpEvent *rdp_event;
 
   rdp_event = g_malloc0 (sizeof (RdpEvent));
   rdp_event->type = RDP_EVENT_TYPE_INPUT_PTR_MOTION_ABS;
   rdp_event->input_ptr_motion_abs.stream = stream;
-  rdp_event->input_ptr_motion_abs.x = x;
-  rdp_event->input_ptr_motion_abs.y = y;
+  rdp_event->input_ptr_motion_abs.motion_abs = *motion_abs;
 
   queue_rdp_event (rdp_event_queue, rdp_event);
 }
@@ -374,24 +379,18 @@ grd_rdp_event_queue_new (GrdSessionRdp *session_rdp)
   return rdp_event_queue;
 }
 
-void
-free_rdp_event (gpointer data)
-{
-  RdpEvent *rdp_event = data;
-
-  g_free (rdp_event);
-}
-
 static void
 grd_rdp_event_queue_dispose (GObject *object)
 {
   GrdRdpEventQueue *rdp_event_queue = GRD_RDP_EVENT_QUEUE (object);
 
   g_clear_pointer (&rdp_event_queue->rdp_sync_event, g_free);
-  g_queue_free_full (rdp_event_queue->queue, free_rdp_event);
 
-  g_source_destroy (rdp_event_queue->flush_source);
-  g_clear_pointer (&rdp_event_queue->flush_source, g_source_unref);
+  if (rdp_event_queue->flush_source)
+    {
+      g_source_destroy (rdp_event_queue->flush_source);
+      g_clear_pointer (&rdp_event_queue->flush_source, g_source_unref);
+    }
 
   G_OBJECT_CLASS (grd_rdp_event_queue_parent_class)->dispose (object);
 }
@@ -403,6 +402,8 @@ grd_rdp_event_queue_finalize (GObject *object)
 
   g_mutex_clear (&rdp_event_queue->event_mutex);
 
+  g_queue_free_full (rdp_event_queue->queue, free_rdp_event);
+
   G_OBJECT_CLASS (grd_rdp_event_queue_parent_class)->finalize (object);
 }
 
