@@ -24,10 +24,6 @@
 #include <glib-object.h>
 
 #include "grd-enums.h"
-#include "grd-types.h"
-
-#define GRD_RDP_SCHEMA_ID "org.gnome.desktop.remote-desktop.rdp"
-#define GRD_VNC_SCHEMA_ID "org.gnome.desktop.remote-desktop.vnc"
 
 typedef struct _GrdSettings GrdSettings;
 
@@ -38,29 +34,9 @@ G_DECLARE_DERIVABLE_TYPE (GrdSettings, grd_settings,
 struct _GrdSettingsClass
 {
   GObjectClass parent_class;
-
-  gboolean (* is_rdp_enabled) (GrdSettings *settings);
-
-  gboolean (* is_vnc_enabled) (GrdSettings *settings);
-
-  gboolean (* get_rdp_view_only) (GrdSettings *settings);
-
-  gboolean (* get_vnc_view_only) (GrdSettings *settings);
-
-  GrdRdpScreenShareMode (* get_rdp_screen_share_mode) (GrdSettings *settings);
-
-  GrdVncScreenShareMode (* get_vnc_screen_share_mode) (GrdSettings *settings);
-
-  char * (* get_rdp_server_cert) (GrdSettings *settings);
-
-  char * (* get_rdp_server_key) (GrdSettings *settings);
-
-  GrdVncAuthMethod (* get_vnc_auth_method) (GrdSettings *settings);
 };
 
-int grd_settings_get_rdp_port (GrdSettings *settings);
-
-int grd_settings_get_vnc_port (GrdSettings *settings);
+GrdRuntimeMode grd_settings_get_runtime_mode (GrdSettings *settings);
 
 void grd_settings_override_rdp_port (GrdSettings *settings,
                                      int          port);
@@ -73,25 +49,24 @@ gboolean grd_settings_get_rdp_credentials (GrdSettings  *settings,
                                            char        **password,
                                            GError      **error);
 
+gboolean grd_settings_set_rdp_credentials (GrdSettings  *settings,
+                                           const char   *username,
+                                           const char   *password,
+                                           GError      **error);
+
+gboolean grd_settings_clear_rdp_credentials (GrdSettings  *settings,
+                                             GError      **error);
+
+void grd_settings_recreate_rdp_credentials (GrdSettings *settings);
+
 char *grd_settings_get_vnc_password (GrdSettings *settings,
                                       GError     **error);
 
-gboolean grd_settings_is_rdp_enabled (GrdSettings *settings);
+gboolean grd_settings_set_vnc_password (GrdSettings  *settings,
+                                        const char   *password,
+                                        GError      **error);
 
-gboolean grd_settings_is_vnc_enabled (GrdSettings *settings);
-
-gboolean grd_settings_get_rdp_view_only (GrdSettings *settings);
-
-gboolean grd_settings_get_vnc_view_only (GrdSettings *settings);
-
-GrdRdpScreenShareMode grd_settings_get_rdp_screen_share_mode (GrdSettings *settings);
-
-GrdVncScreenShareMode grd_settings_get_vnc_screen_share_mode (GrdSettings *settings);
-
-char *grd_settings_get_rdp_server_cert (GrdSettings *settings);
-
-char *grd_settings_get_rdp_server_key (GrdSettings *settings);
-
-GrdVncAuthMethod grd_settings_get_vnc_auth_method (GrdSettings *settings);
+gboolean grd_settings_clear_vnc_password (GrdSettings  *settings,
+                                          GError      **error);
 
 #endif /* GRD_SETTINGS_H */
